Attribute VB_Name = "Conv2XMLUtils"
Option Explicit

' globals used by the Word Converter, not my choice and I'll try and get rid of them sometime

' The XMLConverter is the object used to store variables set by the user and contains
' the actual parsing engine.
Public objConverter As New XMLConverter

' This variable stores the Word document being converted.
Public WordDocument As Word.Document

' This variable stores the text file the XML is output to.
Public XMLFile As Scripting.TextStream
' end of Word globals

Public Function GetDataSourceType(oJobXML As MSXML2.IXMLDOMNode) As DataSourceType
    Select Case (GetNodeText(oJobXML, "data/type"))
    Case "WORD"
        GetDataSourceType = MSWord
        main.LogMessage StatusMessage, "DataSource Type: Word"
    Case "EXCEL"
        GetDataSourceType = MSExcel
        main.LogMessage StatusMessage, "DataSource Type: Excel"
    Case "ORACLE", "SQLSERVER", "ACCESS"
        GetDataSourceType = ADO
        main.LogMessage StatusMessage, "DataSource Type: ADO"
    Case Else
        GetDataSourceType = ADO
        main.LogMessage StatusMessage, "DataSource Type: ADO"
    End Select
End Function

Public Function InitWordXMLConverter(ByVal strSrc As String, ByVal strDest As String, oExtract As MSXML2.IXMLDOMNode) As Boolean
    objConverter.DocumentType = xmlDocumentTypeEnhanced
    objConverter.XMLRecovery = True
    
    objConverter.DocumentProperties = xmlAllProperties
    
    objConverter.StyleType = xmlAllStyles
    objConverter.StyleList = False
    objConverter.StyleListType = xmlInUseList
    objConverter.CSSStyleDescriptions = False
    
    objConverter.GraphicsInfo = xmlDoNotWriteGraphics
    
    objConverter.SaveFileName = strDest
    objConverter.OpenFileName = strSrc
    
    InitWordXMLConverter = True
End Function

Public Sub ExtractXMLFromWord(oExtract As MSXML2.IXMLDOMNode)
    Dim wdApplication As Word.Application
    Dim docWord As Word.Document
    Dim dsiStyles As New DocumentStyleInformation
    Dim dgiGraphics As New DocumentGraphicInformation
    Dim blnWriteGraphics As Boolean
    Dim strFileName As String
    Dim fsoFileScriptingObject As New Scripting.FileSystemObject
    Dim tstXMLFile As Scripting.TextStream
    
    Set wdApplication = CreateObject("Word.Application")
    Set WordDocument = wdApplication.Documents.Open(FileName:=objConverter.OpenFileName, ReadOnly:=True)
    ' Keep Word hidden.
    With wdApplication.ActiveWindow.View
        .ShowAll = False
        .ShowParagraphs = False
        .ShowSpaces = False
        .ShowTabs = False
        .ShowHyphens = False
        .ShowHiddenText = True
    End With
    
    Set tstXMLFile = fsoFileScriptingObject.CreateTextFile(FileName:=objConverter.SaveFileName)

    ' Call the appropriate function based on settings made by the user.
    If objConverter.DocumentType = xmlDocumentTypeBasic Then
        ' Basic output.
        objConverter.ConvertToXML WordDocument:=WordDocument, XMLFile:=tstXMLFile
    Else
        ' Gets graphic information.
        If objConverter.GraphicsInfo = xmlWriteAllGraphics Then
            main.LogMessage StatusMessage, "   Getting graphics information"
            dgiGraphics.GetGraphicInformation WordDocument:=WordDocument, LinkedOnly:=False
            blnWriteGraphics = True
        ElseIf objConverter.GraphicsInfo = xmlWriteLinkedGraphics Then
            main.LogMessage StatusMessage, "   Getting graphics information"
            dgiGraphics.GetGraphicInformation WordDocument:=WordDocument, LinkedOnly:=True
            blnWriteGraphics = True
        End If
        
        ' Gets style information and calls parser (ConvertTOXML).
        If objConverter.StyleType = xmlAllStyles And blnWriteGraphics Then
            main.LogMessage StatusMessage, "   Getting style information"
            dsiStyles.GetStyleInformation WordDocument:=WordDocument, IncludeCSS:=objConverter.CSSStyleDescriptions
            
            ' Parser call when graphics are included.
            objConverter.ConvertToXML WordDocument:=WordDocument, _
                XMLFile:=tstXMLFile, _
                DocStyles:=dsiStyles, _
                Graphics:=dgiGraphics
        
        ElseIf objConverter.StyleType = xmlAllStyles And blnWriteGraphics = False Then
            main.LogMessage StatusMessage, "   Getting style information"
            dsiStyles.GetStyleInformation WordDocument, objConverter.CSSStyleDescriptions
            
            ' Parser call when graphics are excluded.
            objConverter.ConvertToXML WordDocument:=WordDocument, _
                XMLFile:=tstXMLFile, _
                DocStyles:=dsiStyles
        ElseIf objConverter.StyleType <> xmlAllStyles And blnWriteGraphics Then
            
            ' Parser call when styles are excluded and graphics are included.
            objConverter.ConvertToXML WordDocument:=WordDocument, _
                XMLFile:=tstXMLFile, _
                Graphics:=dgiGraphics
        Else
            ' Same as the basic parser call.  The only difference here is that
            ' the user didn't select the basic output option.  Instead they chose
            ' the enhanced output option, but didn't change any of the defaults
            ' as they stepped through the wizard.
            objConverter.ConvertToXML WordDocument:=WordDocument, _
                XMLFile:=tstXMLFile
        End If
    End If

    ' Closes out all variables.
    WordDocument.Close savechanges:=wdDoNotSaveChanges
    Set tstXMLFile = Nothing
    Set fsoFileScriptingObject = Nothing
    Set WordDocument = Nothing
    wdApplication.Quit
    Set wdApplication = Nothing
End Sub

Public Function GetExtractCommand(ds_type As DataSourceType, oExtract As MSXML2.IXMLDOMNode) As String
    Dim strTemp As String
    strTemp = GetNodeText(oExtract, "command")
    If strTemp <> "" Then
        GetExtractCommand = strTemp
        Exit Function
    End If
    
    strTemp = GetNodeText(oExtract, "table")
    Select Case ds_type
    Case MSExcel
        GetExtractCommand = "SELECT * FROM `" & strTemp & "$`"
    Case Else
        GetExtractCommand = "SELECT * FROM " & strTemp
    End Select
End Function

Public Sub ExtractXMLFromADO(ByVal strProvider As String, ByVal strConnection As String, ByVal strCommand As String, ByVal strXMLFile As String)
    'connection
    Dim objConnect As ADODB.Connection
    Set objConnect = CreateObject("ADODB.Connection")
    With objConnect
        .Provider = strProvider
        .ConnectionString = strConnection
        .Open
    End With
    
    'command
    Dim objCommand As ADODB.Command
    Set objCommand = CreateObject("ADODB.Command")
    With objCommand
        .CommandText = strCommand
        .CommandType = adCmdText
    End With
    
    'results
    Dim rsResults As ADODB.Recordset
    Set rsResults = CreateObject("ADODB.Recordset")
    With rsResults
        .CursorLocation = adUseClient
        .CursorType = adOpenStatic
        .LockType = adLockReadOnly
    End With
    
    'execute the command on the connection and pour into the results
    objCommand.ActiveConnection = objConnect
    objCommand.Execute
    rsResults.Open objCommand
    
    'save as XML
    If Dir(strXMLFile) <> "" Then Kill strXMLFile
    rsResults.Save strXMLFile, adPersistXML
    
    ' wake up, time to die
    objConnect.Close
    
    If Not rsResults Is Nothing Then Set rsResults = Nothing
    If Not objConnect Is Nothing Then Set objConnect = Nothing
    If Not objCommand Is Nothing Then Set objCommand = Nothing
End Sub

Public Sub ExtractTable(ByVal strProvider As String, ByVal strConnect As String, ByVal strXML As String, ByVal strTable As String)

    ExtractXMLFromADO strProvider, strConnect, strTable & "$", strXML & strTable & ".xml"

End Sub

Public Sub ProcessTable(ByVal strSrc As String, ByVal strXSLT As String, ByVal strDest As String, Optional parrParams As Variant)
    
    Dim oResult As MSXML2.DOMDocument
    Set oResult = TransformXMLFromFile(strSrc, strXSLT, parrParams)
    oResult.Save (strDest)
    Set oResult = Nothing
    
End Sub


