Attribute VB_Name = "HTTPUtils"
Option Explicit

Private Const scCLASS_NAME = "GlobalWatches.HTTPUtils."
Private Const scComponentName = "GlobalWatches"


' The ServerXMLHTTP object requires MSXML 3.0 SP1 to be installed
' If you are getting "access denied" errors ensure you have
' run the proxycfg.exe utility.  For details see :
' http://support.microsoft.com/support/kb/articles/q290/7/61.asp
' http://support.microsoft.com/support/kb/articles/q289/4/81.asp
' http://support.microsoft.com/support/kb/articles/q291/8/41.asp
' Utility should be available in SourceSafe, or alternatively :
' http://msdn.microsoft.com/code/sample.asp?url=/msdn-files/027/001/468/msdncompositedoc.xml


' Comms timeout values in milliseconds

Private Const clngRESOLVE As Long = 30000
Private Const clngCONNECT As Long = 30000
Private Const clngSEND As Long = 30000
Private Const clngRECEIVE As Long = 30000


'********************************************************************************
' METHOD NAME:  GetURL
'
' INPUTS:       ptURL   string containing the URL of the data
'
' RETURNS:      string containing the data returned
'
' PURPOSE:      Retrieves data from a URL in a server safe way.  NB this function
'               is only safe for 7 bit ASCII characters, 8 bit characters may not
'               always be returned successfully.
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Public Function GetURL(ptURL As String) As String
    Dim objXmlHttp As New MSXML2.ServerXMLHTTP

    On Error GoTo ErrorHandler

    objXmlHttp.setTimeouts clngRESOLVE, clngCONNECT, clngSEND, clngRECEIVE
    objXmlHttp.open "GET", ptURL, False
    objXmlHttp.send
    GetURL = objXmlHttp.responseText

ExitMethod:
    Exit Function

ErrorHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "GetURL"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "GetURL : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function


'********************************************************************************
' METHOD NAME:  GetBinaryURL
'
' INPUTS:       ptURL   string containing the URL of the data
'
' RETURNS:      variant array of bytes containing the data returned
'
' PURPOSE:      Retrieves data from a URL in a server safe way.
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Function GetBinaryURL(ptURL As String) As Variant
    Dim objXmlHttp As New MSXML2.ServerXMLHTTP

    On Error GoTo ErrorHandler

    objXmlHttp.setTimeouts clngRESOLVE, clngCONNECT, clngSEND, clngRECEIVE
    objXmlHttp.open "GET", ptURL, False
    objXmlHttp.send
    GetBinaryURL = objXmlHttp.responseBody
    Set objXmlHttp = Nothing

ExitMethod:
    Exit Function

ErrorHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "GetBinaryURL"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "GetBinaryURL : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function


'********************************************************************************
' METHOD NAME:  GetXMLURL
'
' INPUTS:       ptURL   string containing the URL of the data
'
' RETURNS:      XML document object containing a parsed version of the data returned
'
' PURPOSE:      Retrieves data from a URL in a server safe way.  NB this function
'               will only work if the content type of the retrieved data is set
'               to text/xml, otherwise an empty XML object is returned.  Domino does
'               not use the correct type when retrieving HTML documents from views.
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Function GetXMLURL(ptURL As String) As MSXML2.DOMDocument
    Dim objXmlHttp As New MSXML2.ServerXMLHTTP

    On Error GoTo ErrorHandler

    objXmlHttp.setTimeouts clngRESOLVE, clngCONNECT, clngSEND, clngRECEIVE
    objXmlHttp.open "GET", ptURL, False
    objXmlHttp.send
    Set GetXMLURL = objXmlHttp.responseXML

ExitMethod:
    Exit Function

ErrorHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "GetXMLURL"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "GetXMLURL : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function


'********************************************************************************
' METHOD NAME:  GetXMLURL
'
' INPUTS:       ptURL   string containing the URL of the data
'
' RETURNS:      an IStream interface pointer
'
' PURPOSE:      Retrieves data from a URL in a server safe way.
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Function GetStreamURL(ptURL As String) As Variant
    Dim objXmlHttp As New MSXML2.ServerXMLHTTP

    On Error GoTo ErrorHandler

    objXmlHttp.setTimeouts clngRESOLVE, clngCONNECT, clngSEND, clngRECEIVE
    objXmlHttp.open "GET", ptURL, False
    objXmlHttp.send
    Set GetStreamURL = objXmlHttp.responseStream

ExitMethod:
    Exit Function

ErrorHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "GetStreamURL"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "GetStreamURL : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function

