Attribute VB_Name = "SortRoutine"
Option Explicit


' This function uses Microsoft Excel to sort arrays by the selected column in the array.
' Rather than writing a custom sort routine, I chose to take advantage of the power
' of the applications in Office.
'
' PARAMETERS:
' varOldArray: This is the array to be sorted.
' intSortField: This represents the field in the array on which to sort.
Function ExcelSort(varOldArray As Variant, intSortField As Integer) As Variant
    Dim xlApp As Excel.Application
    Dim wbTemp As Excel.Workbook
    Dim shtTemp As Excel.Worksheet
    Dim lngRowCounter As Long ' This stores the current row in the array.
    Dim lngColCounter As Long ' This stores the current column in the array.
    Dim lngColumns As Long ' This represents the number of columns in the array.
    Dim lngRows As Long ' This represents the number of rows in the array.
    Dim rngSortField As Excel.Range
    
    Set xlApp = CreateObject("Excel.Application")
    xlApp.Visible = False
    Set wbTemp = xlApp.Workbooks.Add
    Set shtTemp = wbTemp.Worksheets(1)
    
    ' Get the dimensions of the array.
    lngRows = UBound(varOldArray, 2) - 1
    lngColumns = UBound(varOldArray, 1) - 1
    
    ' This populates a spreadsheet based on the dimensions of the array,
    ' with the contents of the array.
    For lngRowCounter = 0 To lngRows
        For lngColCounter = 0 To lngColumns
            shtTemp.Range("A1").Offset(RowOffset:=lngRowCounter, ColumnOffset:=lngColCounter).Value = _
                varOldArray(lngColCounter + 1, lngRowCounter + 1)
        Next
    Next
    
    Set rngSortField = shtTemp.Range("A1").Offset(RowOffset:=0, ColumnOffset:=intSortField - 1)
    
    ' Sort the array.
    shtTemp.Range(shtTemp.Range("A1"), shtTemp.Range("A1").Offset(lngRows, lngColumns)).Sort key1:=rngSortField, order1:=xlAscending, header:=xlNo
    
    ' Repopulate the array with the sorted data.
    For lngRowCounter = 0 To lngRows
        For lngColCounter = 0 To lngColumns
        varOldArray(lngColCounter + 1, lngRowCounter + 1) = shtTemp.Range("A1").Offset(RowOffset:=lngRowCounter, ColumnOffset:=lngColCounter).Value
        Next
    Next
    
    ' Close out the variables.
    wbTemp.Close False
    xlApp.Quit
    Set wbTemp = Nothing
    Set xlApp = Nothing
    
    ' Return the sorted array.
    ExcelSort = varOldArray
    
End Function
