VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "XMLConverter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit
' Public Enumerations:
'    XMLDocumentType: This is used to identify what type of XML document is produced.
'    XMLDocProperties: This describes what set of Word document properties are included
'       in the XML output.
'    XMLStyleType: This describes which style set is used in the XML output.
'    XMLGraphicsInfo: This describes how graphics are handled.
'    XMLStyleList: This describes which styles are written out (all or used only).
'
' Public Properties:
'    OpenFileName: Sets or returns a String value representing the Word document to be converted.
'    SafeFileName: Sets or returns a String value of the name to save the new XML file to.
'    DocumentType: Sets or returns a XMLDocumentType value that defines the XML output.
'    StyleType: Sets or returns a XMLStyleType value that defines the style output.
'    DocumentProperties: Sets or returns XMLDocProperties value that defines what properties to write.
'    WriteStatus: Sets or returns a Long value representing the current status of the current XML output.
'    StyleList: Sets or returns a Boolean value that defines whether to include style list.
'    StyleListType: Sets or returns a XMLStyleList value that defines what styles to output.
'    GraphicsInfo: Sets or returns a XMLGraphicsInfo value that defines what graphics to write out.
'    CSSStyleDescriptions: Sets or returns a Boolean that defines whether to include CSS information.
'    CreateHTMandXSLFiles: Sets or returns a Boolean that defines whether to write HTM and XSL file.
'    XMLRecovery: Sets or returns a Boolean value defines whether to save errors to the XML output.
'
' Public Methods:
'    ConvertToXML, This handles parsing the Word document to XML.
'    PARAMETERS:
'        WordDocument: Word.Document, document being converted
'        XMLFile: Scripting.TextStream, XML output file
'        OPTIONAL:
'        DocStyles: DocumentStyleInformation, StylesInstances collection
'        Graphics: DocumentGraphicInformation, GraphicInstances collecton
'
'    CleanString, removes inappropriate characters, returns String
'    PARAMETERS:
'        strDirty: String, string to convert
'        OPTIONAL:
'        TextTags: Boolean, if TRUE wraps return string in <TEXT> tags.
'
'    WriteShape, writes out Shape XML, returns String
'    PARAMETERS:
'        shpCurrent, Word.Shape, Shape being written
'
'    WriteInlineShape, writes out InlineShape XML, returns String
'    PARAMETERS:
'        ilsCurrent, Word.InlineShape, InlineShape being written
'
'    WriteGraphicInfo, writes out graphic XML, returns String
'    PARAMETERS:
'        giObject, GraphicInstance, graphic being written
'
'    ParseForGraphics, checks a paragraph range for graphics, returns String
'    PARAMETERS:
'        WordDocument, Word.Document, document being converted
'        dgiCurrent, DocumentGraphicInformation, collection of document graphics
'        curRange, Word.Range, current range
'        OPTIONAL:
'        blnStyledText, Boolean, whether the range contains styled text
'
'    GetCSSArray, gets an array of the CSS descriptions of the styles, returns Variant
'    PARAMETERS:
'        docTemp, Word.Document, the document being converted
'
'    ParseTable, parses a table to XML, returns String
'    PARAMETERS:
'        tblCurrent, Word.Table, table being parsed
'        OPTIONAL:
'        DocStyles, DocumentStyleInformation
'        Graphics, DocumentGraphicInformation
'
'    ParseCell, parses contents of individual cells, returns String
'    PARAMETERS:
'        celActive, Word.Cell, cell being parsed
'        OPTIONAL:
'        DocStyles, DocumentStyleInformation, document styles
'        Graphics, DocumentGraphicInformation, document graphics
'
'    ParseParagraph, parses individual paragraphs, returns String
'    PARAMETERS:
'        parCounter, Word.Paragraph, paragraph to parse
'        OPTIONAL:
'        DocStyles, DocumentStyleInformation, document styles
'        Graphics, DocumentGraphicInformation, document graphics
'
Public Enum XMLDocumentType
    xmlDocumentTypeBasic = 0
    xmlDocumentTypeEnhanced = 1
End Enum

Public Enum XMLDocProperties
    xmlNoProperties = 0
    xmlAllProperties = 1
    xmlBuiltInOnly = 2
    xmlCustomOnly = 3
End Enum

Public Enum XMLStyleType
    xmlAllStyles = 0
    xmlBaseStylesOnly = 1
End Enum

Public Enum XMLGraphicsInfo
    xmlDoNotWriteGraphics = 0
    xmlWriteLinkedGraphics = 1
    xmlWriteAllGraphics = 2
    xmlWriteGraphicsList = 3
End Enum
Public Enum XMLStyleList
    xmlInUseList = 0
    xmlCompleteList = 1
End Enum


Private mvarOpenFileName As String
Private mvarSaveFileName As String
Private mvarDocumentType As XMLDocumentType
Private mvarStyleType As XMLStyleType
Private mvarDocumentProperties As XMLDocProperties
Private mvarWriteStatus As Long
Private mvarStyleList As Boolean
Private mvarStyleListType As XMLStyleList
Private mvarGraphicsInfo As XMLGraphicsInfo
Private mvarCSSStyleDescriptions As Boolean
Private mvarXMLRecovery As Boolean

Public Property Let XMLRecovery(ByVal vData As Boolean)
    
    mvarXMLRecovery = vData

End Property

Public Property Get XMLRecovery() As Boolean
    
    XMLRecovery = mvarXMLRecovery

End Property

Public Property Let CSSStyleDescriptions(ByVal vData As Boolean)
    
    mvarCSSStyleDescriptions = vData

End Property

Public Property Get CSSStyleDescriptions() As Boolean
    
    CSSStyleDescriptions = mvarCSSStyleDescriptions

End Property

Public Property Let GraphicsInfo(ByVal vData As XMLGraphicsInfo)
    
    mvarGraphicsInfo = vData

End Property

Public Property Get GraphicsInfo() As XMLGraphicsInfo
    
    GraphicsInfo = mvarGraphicsInfo

End Property

Public Property Let StyleListType(ByVal vData As XMLStyleList)
    
    mvarStyleListType = vData

End Property

Public Property Get StyleListType() As XMLStyleList
    
    StyleList = mvarStyleListType

End Property

Public Property Let StyleList(ByVal vData As Boolean)
    
    mvarStyleList = vData

End Property

Public Property Get StyleList() As Boolean
    
    StyleList = mvarStyleList

End Property

Public Property Let WriteStatus(ByVal vData As Long)
    
    mvarWriteStatus = vData

End Property

Public Property Get WriteStatus() As Long
    
    WriteStatus = mvarWriteStatus

End Property

Public Property Let DocumentProperties(ByVal vData As XMLDocProperties)
    
    mvarDocumentProperties = vData

End Property

Public Property Get DocumentProperties() As XMLDocProperties
    
    DocumentProperties = mvarDocumentProperties

End Property

Public Property Let StyleType(ByVal vData As XMLStyleType)
    
    mvarStyleType = vData

End Property

Public Property Get StyleType() As XMLStyleType
    
    StyleType = mvarStyleType

End Property

Public Property Let DocumentType(ByVal vData As XMLDocumentType)
    
    mvarDocumentType = vData

End Property

Public Property Get DocumentType() As XMLDocumentType
    
    DocumentType = mvarDocumentType

End Property

Public Property Let SaveFileName(ByVal vData As String)
    
    mvarSaveFileName = vData

End Property

Public Property Get SaveFileName() As String
    
    SaveFileName = mvarSaveFileName

End Property

Public Property Let OpenFileName(ByVal vData As String)
    
    mvarOpenFileName = vData

End Property

Public Property Get OpenFileName() As String
    
    OpenFileName = mvarOpenFileName

End Property

Public Sub ConvertToXML(WordDocument As Word.Document, XMLFile As Scripting.TextStream, Optional DocStyles As DocumentStyleInformation, Optional Graphics As DocumentGraphicInformation)
' Main routine that handles XML conversion.

    Const ERR_BADPROPERTY      As Long = 5
    Const ERR_BADDOCOBJ        As Long = 438
    Const ERR_BADCONTEXT       As Long = -2147467259
    
    Dim parCounter As Word.Paragraph
    Dim wParas As Word.Paragraphs
    Dim intCounter As Integer 'This is used simply as a counter in For Next loops.
    Dim intStatusCounter As Integer 'This is used to change the value of the progress bar.
    Dim stlCurrent As Style
    Dim colDocProperties As Object 'This stores the document properties.
    Dim strNodeName As String 'This stores the text of the XML node currently being written.
    Dim dgiCurrent As DocumentGraphicInformation
    Dim dsiCurrent As DocumentStyleInformation
    Dim giCurrent As GraphicInstance
    Dim giNext As GraphicInstance 'This stores the graphic in the sequence that occurs after the one currently being written.
    Dim siCurrent As StyleInstance 'This stores the style in the sequence that occurs after the one currently being written.
    Dim siNext As StyleInstance
    Dim blnResumeParagraphProcessing As Boolean 'This is a flag used to determine whether or not to continue processing paragraphs.
    Dim blnTableComplete As Boolean 'This is a flag used to determine whether the end of a table has been reached.
    Dim blnInList As Boolean 'This is a flag used to determine whether execution is inside a list.
    Dim tblCurrent As Word.Table
    Dim rowCurrent As Word.Row
    Dim celCurrent As Word.Cell
    Dim strRow As String 'This represents a row of data in a table.
    Dim intListType As String 'This represents the type of list being written.
    Dim varCSSArray() As Variant 'This stores an array of strings representing the CSS formatting for styles.
    Dim strCSS As String 'This represents the CSS format for the style currently being written.
    Dim docTemp As New Word.Document
    Dim ilsCurrent As Word.InlineShape
    Dim shpCurrent As Word.Shape
    Dim strTable As String 'This represents the XML string for a table.
    Dim intParaCount As Integer 'This represents the total paragraphs in the document being converted.
    
    intStatusCounter = 0
    intParaCount = WordDocument.Paragraphs.Count

On Error GoTo ErrHandler
    Set wParas = WordDocument.Paragraphs
    
    ' This writes the initial XML declaration.
    XMLFile.WriteLine "<?xml version=""1.0"" encoding=""ISO-8859-1""?>"
    
    ' This writes the document root
    XMLFile.WriteLine "<Document name=" & Chr(34) & WordDocument.FullName & Chr(34) & ">"
    
    ' Check for XML document type (basic or enhanced).
    If objConverter.DocumentType = xmlDocumentTypeBasic Then
        XMLFile.WriteLine "<DocumentBody>"
            For Each parCounter In wParas
                intStatusCounter = intStatusCounter + 1
                If intStatusCounter Mod 10 = 0 Then
                    main.LogMessage StatusMessage, "    Writing paragraph " & intStatusCounter & " of " & intParaCount
                End If
                XMLFile.WriteLine "<Paragraph>" & CleanString(parCounter.Range, True) & "</Paragraph>"
            Next
    ElseIf objConverter.DocumentType = xmlDocumentTypeEnhanced Then
    
        ' Write property information.
        If objConverter.DocumentProperties <> xmlNoProperties Then
            XMLFile.WriteLine "<DocumentProperties>"
            
            main.LogMessage StatusMessage, "   Writing document properties..."
            
            ' Determine what properties to write out.
            Select Case objConverter.DocumentProperties
                Case xmlAllProperties
            
                    XMLFile.WriteLine "<BuiltInProperties>"
                        Set colDocProperties = WordDocument.BuiltinDocumentProperties
                    
                        For intCounter = 1 To colDocProperties.Count
                            strNodeName = colDocProperties(intCounter).Name
                            XMLFile.WriteLine "<Property name=" & Chr(34) & strNodeName & Chr(34) & ">" & colDocProperties(intCounter).Value & "</Property>"
                        Next
                    XMLFile.WriteLine "</BuiltInProperties>"
                    
                    XMLFile.WriteLine "<CustomProperties>"
                        Set colDocProperties = WordDocument.CustomDocumentProperties
                    
                        For intCounter = 1 To colDocProperties.Count
                            strNodeName = colDocProperties(intCounter).Name
                            XMLFile.WriteLine "<Property name=" & Chr(34) & strNodeName & Chr(34) & ">" & colDocProperties(intCounter).Value & "</Property>"
                        Next
                        
                    XMLFile.WriteLine "</CustomProperties>"
    
                Case xmlBuiltInOnly
                    XMLFile.WriteLine "<BuiltInProperties>"
                        Set colDocProperties = WordDocument.BuiltinDocumentProperties
                    
                        For intCounter = 1 To colDocProperties.Count
                            strNodeName = colDocProperties(intCounter).Name
                            If Len(strNodeName) > 0 Then
                                XMLFile.WriteLine "<Property name=" & Chr(34) & strNodeName & Chr(34) & ">" & colDocProperties(intCounter).Value & "</Property>"
                            End If
                        Next
                    XMLFile.WriteLine "</BuiltInProperties>"
                Case xmlCustomOnly
                    XMLFile.WriteLine "<CustomProperties>"
                        Set colDocProperties = WordDocument.CustomDocumentProperties
                    
                        For intCounter = 1 To colDocProperties.Count
                            strNodeName = colDocProperties(intCounter).Name
                            If Len(strNodeName) > 0 Then
                                XMLFile.WriteLine "<Property name=" & Chr(34) & strNodeName & Chr(34) & ">" & colDocProperties(intCounter).Value & "</Property>"
                            End If
                        Next
                    XMLFile.WriteLine "</CustomProperties>"
            End Select
            XMLFile.WriteLine "</DocumentProperties>"
        End If
    
        ' Write document style information.
        If objConverter.StyleList Then
            main.LogMessage StatusMessage, "   Writing document styles..."
            
            ' Get CSS information if required.
            If objConverter.CSSStyleDescriptions Then
                WordDocument.Range.Copy
                docTemp.Range.Paste
                varCSSArray = GetCSSArray(docTemp)
            End If
                            
            If objConverter.CSSStyleDescriptions Then
                XMLFile.WriteLine "<DocumentStyles CSS=" & Chr(34) & "True" & Chr(34) & ">"
            Else
                XMLFile.WriteLine "<DocumentStyles>"
            End If
            intStatusCounter = 0
'            frmWizard.prbStatus.Max = WordDocument.Styles.Count
            For Each stlCurrent In WordDocument.Styles
            
                ' Update form with style status.
                intStatusCounter = intStatusCounter + 1
'                With frmWizard
'                    .prbStatus.Value = intStatusCounter
'                    .Refresh
'                End With
                
                ' Get CSS style descriptions.
                If objConverter.CSSStyleDescriptions Then
                    For intCounter = 1 To UBound(varCSSArray, 2)
                        If InStr(1, stlCurrent.NameLocal, varCSSArray(1, intCounter)) Then
                            strCSS = varCSSArray(2, intCounter)
                            Exit For
                        End If
                    Next
                End If
                    
                ' Write out style information.
                If objConverter.StyleListType = xmlCompleteList Or _
                    objConverter.StyleListType = xmlInUseList And _
                    stlCurrent.InUse Then
                    
                    XMLFile.WriteLine "<Style name=" & Chr(34) & Replace(Expression:=stlCurrent.NameLocal, Find:=Chr(32), Replace:="") & Chr(34) & ">"
                    XMLFile.WriteLine "<StyleDescription>" & stlCurrent.Description & "</StyleDescription>"
                    If objConverter.CSSStyleDescriptions And Len(strCSS) > 0 Then
                        XMLFile.WriteLine "<StyleCSSDescription>" & strCSS & "</StyleCSSDescription>"
                        strCSS = ""
                    End If
                    XMLFile.Write "</Style>"
                End If
            Next
            XMLFile.WriteLine "</DocumentStyles>"
        End If
        intStatusCounter = 0
        
        ' Write body of document.
        XMLFile.WriteLine "<DocumentBody>"
        
        Select Case objConverter.StyleType
            
            ' Write paragraph by paragraph, no styles, graphics, tables, or lists.
            Case xmlBaseStylesOnly
                For Each parCounter In wParas
                    intStatusCounter = intStatusCounter + 1
                    If intStatusCounter Mod 10 = 0 Then
                        main.LogMessage StatusMessage, "    Writing paragraph " & intStatusCounter & " of " & intParaCount
                    End If
                    XMLFile.WriteLine "<Paragraph StyleName=" & Chr(34) & Replace(parCounter.Style, Chr(32), "") & Chr(34) & ">" & CleanString(parCounter.Range, True) & "</Paragraph>"
                Next
            ' Write out all style information, graphics, tables, lists.
            Case xmlAllStyles
                
                For Each parCounter In wParas
                    intStatusCounter = intStatusCounter + 1
                    If intStatusCounter Mod 10 = 0 Then
                        main.LogMessage StatusMessage, "    Writing paragraph " & intStatusCounter & " of " & intParaCount
                    End If
                    ' Handle Tables. If a table exists, control is passed
                    ' to ParseTable.
                    If parCounter.Range.Tables.Count = 0 Then
                        blnResumeParagraphProcessing = True
                        blnTableComplete = False
                    ElseIf blnTableComplete = False Then
                        Set tblCurrent = parCounter.Range.Tables(1)
                        strTable = ParseTable(tblCurrent:=tblCurrent, DocStyles:=DocStyles, Graphics:=Graphics)
                        XMLFile.WriteLine strTable
                        blnTableComplete = True
                        blnResumeParagraphProcessing = False
                    End If


                    ' Handle Lists. If a list exists, output is handled here.
                    If parCounter.Range.ListParagraphs.Count > 0 Then
                        If blnInList = False Then
                            XMLFile.WriteLine "<Paragraph>"
                            intListType = parCounter.Range.ListFormat.ListType
                            XMLFile.WriteLine "<List Type=" & _
                                Chr(34) & intListType & Chr(34) & _
                                " StyleName=" & Chr(34) & Replace(Expression:=parCounter.Range.Style, Find:=Chr(32), Replace:="") & Chr(34) & ">"
                        End If

                        With parCounter.Range
                            If intListType = 0 Or intListType = 2 Then
                                XMLFile.WriteLine "<ListItem StyleName=" & Chr(34) & Replace(.Style, Chr(32), "") & Chr(34) & ">" & _
                                        .Text & "</ListItem>"
                            Else
                                ' Even if a list is an outline or numbered, it may have bullets.
                                ' This traps for bulleted characters so garbage characters don't get
                                ' written out. If you use fonts other than Symbol or WingDings, you
                                ' would need to change this If test.
                                If .ListFormat.ListTemplate.ListLevels(.ListFormat.ListLevelNumber).Font.Name = "Symbol" Or _
                                    .ListFormat.ListTemplate.ListLevels(.ListFormat.ListLevelNumber).Font.Name = "WingDings" Then
                                    XMLFile.WriteLine "<ListItem StyleName=" & Chr(34) & Replace(.Style, Chr(32), "") & Chr(34) & _
                                            " Level=" & Chr(34) & _
                                            .ListFormat.ListLevelNumber & Chr(34) & ">" & _
                                            .Text & "</ListItem>"
                                Else
                                    XMLFile.WriteLine "<ListItem StyleName=" & Chr(34) & Replace(.Style, Chr(32), "") & Chr(34) & _
                                            " Level=" & Chr(34) & _
                                            .ListFormat.ListLevelNumber & Chr(34) & ">" & _
                                            .ListFormat.ListString & " " & _
                                            .Text & "</ListItem>"
                                End If
                            End If
                        End With
                        blnInList = True
                        blnResumeParagraphProcessing = False
                        
                        If parCounter.Range = WordDocument.Paragraphs.Last.Range Then
                            XMLFile.WriteLine "</List>"
                            XMLFile.WriteLine "</Paragraph>"
                        End If
                        
                    ElseIf parCounter.Range.ListParagraphs.Count = 0 And blnInList = True Then
                        XMLFile.WriteLine "</List>"
                        XMLFile.WriteLine "</Paragraph>"
                        blnInList = False
                    End If

                    ' Handle paragraph XML output
                    If blnResumeParagraphProcessing Then
                        
                        ' This is a speed enhancement.  If it's a blank line paragraph, doesn't
                        ' do anything but print out a paragraph element.
                        If parCounter.Range.End - parCounter.Range.Start = 1 Then
                            XMLFile.WriteLine "<Paragraph/>"
                        Else
                            XMLFile.WriteLine ParseParagraph(parCounter:=parCounter, DocStyles:=DocStyles, Graphics:=Graphics)
                        End If
                    End If
                Next
        End Select
    
    End If
    
    XMLFile.WriteLine "</DocumentBody>"
    
    ' If the user selected to put graphics in a list, rather than inline,
    ' this section is processed.
    If GraphicsInfo = xmlWriteGraphicsList Then
        XMLFile.WriteLine "<DocumentGraphics>"
        main.LogMessage StatusMessage, "   Writing graphical information..."
        If WordDocument.Shapes.Count > 0 Then
            XMLFile.WriteLine "<Shapes Count=" & Chr(34) & WordDocument.Shapes.Count & Chr(34) & ">"
            For Each shpCurrent In WordDocument.Shapes
                XMLFile.WriteLine WriteShape(shpCurrent)
            Next
            XMLFile.WriteLine "</Shapes>"
        End If
        
        If WordDocument.InlineShapes.Count > 0 Then
            XMLFile.WriteLine "<InlineShapes Count=" & Chr(34) & WordDocument.InlineShapes.Count & Chr(34) & ">"
            For Each ilsCurrent In WordDocument.InlineShapes
                XMLFile.WriteLine WriteInlineShape(ilsCurrent)
            Next
            XMLFile.WriteLine "</InlineShapes>"
        End If
        
        XMLFile.WriteLine "</DocumentGraphics>"
    End If
    
    XMLFile.WriteLine "</Document>"
    XMLFile.Close
ConvertToXML_End:
    Exit Sub
ErrHandler:
    Select Case Err.Number
        ' These first two errors occur when a document property is accessed that
        ' hasn't been populated with information.
        Case ERR_BADPROPERTY
            Resume Next
        Case ERR_BADCONTEXT
            Resume Next
        Case Else
            If objConverter.XMLRecovery Then
                XMLFile.WriteLine "<Error>" & Err.Number & ": " & Err.Description & "</Error>"
                Resume Next
            Else
                XMLFile.Close
                WordDocument.Application.Quit False
                MsgBox prompt:="An error occurred.", _
                    Buttons:=vbOKOnly, _
                    Title:="Error!"
                Unload main
                End
            End If
   End Select
   Resume ConvertToXML_End:

End Sub

' Remove problem characters from strings.
Function CleanString(strDirty As String, Optional TextTags As Boolean = False) As String
    Dim strCleaner As String 'This stores the string as it is being "cleaned".
    Dim intCounter As Integer 'This is used for looping through sets of characters.
    
    ' Replace ampersand, < and > with their appropriate escape sequences.
    strCleaner = Replace(Expression:=strDirty, Find:="&", Replace:="&#38;")
    strCleaner = Replace(Expression:=strCleaner, Find:="<", Replace:="&#60;")
    strCleaner = Replace(Expression:=strCleaner, Find:="/", Replace:="&#47;")
    
    'Characters
    ' replaces unused characters
    For intCounter = 1 To 8
        strCleaner = Replace(Expression:=strCleaner, Find:=Chr(intCounter), Replace:="")
    Next
    For intCounter = 11 To 12
        strCleaner = Replace(Expression:=strCleaner, Find:=Chr(intCounter), Replace:="")
    Next
    For intCounter = 14 To 31
        strCleaner = Replace(Expression:=strCleaner, Find:=Chr(intCounter), Replace:="")
    Next
    
    ' Characters from 130 to 153 are from the Windows character set and don't always
    ' convert well to XML.  These characters here fall within that set and are replaced by
    ' their corresponding Unicode characters.
    
    ' Replace base line single quote (130) with comma (44).
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(130), Replace:=Chr(44))
    
    ' Replace ellipsis (133) with three periods.
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(133), Replace:="...")
    
    ' Replace left (145) and right (146) single quotation marks with an apostrophe (39).
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(145), Replace:=Chr(39))
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(146), Replace:=Chr(39))
    
    ' Replace left (147) and right (148) double quotation marks with a double quote (34).
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(147), Replace:=Chr(34))
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(148), Replace:=Chr(34))
    
    ' Replace a bullet (149) with a hypen (45).
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(149), Replace:=Chr(45))
    
    ' Replace an en (150) and em (151) dash with hyphen (45).
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(150), Replace:=Chr(45))
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(151), Replace:=Chr(45))
    
    ' Replace trademark character (153) with the letters "TM".
    strCleaner = Replace(Expression:=strCleaner, Find:=Chr(153), Replace:="TM")
    
    If TextTags = False Then
        CleanString = strCleaner
    Else
        CleanString = "<Text>" & strCleaner & "</Text>"
    End If
End Function

Function WriteShape(shpCurrent As Word.Shape) As String
' This gets the string for a Shape object representation.

    ' Handle all shapes but AutoShapes.
    If shpCurrent.Type <> msoAutoShape Then
        WriteShape = "<Shape Type=" & Chr(34) & shpCurrent.Type & Chr(34) & _
            " StartPosition=" & Chr(34) & shpCurrent.Anchor.Start & Chr(34) & "/>"
            
    ' Handle Autoshapes.
    ElseIf shpCurrent.Type = msoAutoShape Then
        WriteShape = "<Shape Type=" & Chr(34) & shpCurrent.Type & Chr(34) & _
        " StartPosition=" & Chr(34) & shpCurrent.Anchor.Start & Chr(34) & _
        " AutoShapeType=" & Chr(34) & shpCurrent.AutoShapeType & Chr(34) & "/>"
    End If
End Function

Function WriteInlineShape(ilsCurrent As Word.InlineShape) As String
' This gets a string for an InLineShape object representation.
    
    ' Handle linked objects.
    If ilsCurrent.Type = wdInlineShapeLinkedOLEObject Then
        WriteInlineShape = "<InlineShape Type=" & Chr(34) & _
            ilsCurrent.OLEFormat.ClassType & Chr(34) & _
            " StartPosition=" & Chr(34) & ilsCurrent.Range.Start & Chr(34) & ">" _
            & ilsCurrent.LinkFormat.SourceFullName & _
            "</InlineShape>"
            
    ' Handle embedded objects.
    ElseIf ilsCurrent.Type = wdInlineShapeEmbeddedOLEObject Then
        WriteInlineShape = "<InlineShape Type=" & Chr(34) & ilsCurrent.OLEFormat.ClassType & Chr(34) & _
            " StartPosition=" & Chr(34) & ilsCurrent.Range.Start & Chr(34) & "/>"
    
    ' Handle clipart/pictures.
    ElseIf ilsCurrent.Type = wdInlineShapePicture Then
        WriteInlineShape = "<InlineShape Type=" & Chr(34) & "Unknown picture format" & Chr(34) & _
            " StartPosition=" & Chr(34) & ilsCurrent.Range.Start & Chr(34) & "/>"
    End If
End Function
Function WriteGraphicInfo(giObject As GraphicInstance) As String
' This uses a GraphicInstance object to get graphic information.
    
    Dim strResult As String 'This stores the XML representation of a graphic.
    Dim strTarget As String 'This stores the URL target of a graphic.
    
    ' Write out URL if hyperlink.
    If Len(giObject.Target) > 0 Then
        strTarget = " target=" & Chr(34) & giObject.Target & Chr(34)
    Else
        strTarget = ""
    End If
    
    If giObject.WordShapeType = giInlineShape Then
        
        Select Case giObject.GraphicType
            Case 1 'wdInlineShapeEmbeddedOLEObject
                strResult = "<InlineShape Type=" & Chr(34) & _
                    giObject.OLEClass & Chr(34) & _
                    " StartPosition=" & Chr(34) & giObject.StartPosition & Chr(34) & _
                    " EndPosition=" & Chr(34) & giObject.EndPosition & Chr(34) & strTarget & "/>"
            Case 2 'wdInlineShapeLinkedOLEObject
                strResult = "<InlineShape Type=" & Chr(34) & _
                    giObject.OLEClass & Chr(34) & _
                    " StartPosition=" & Chr(34) & giObject.StartPosition & Chr(34) & _
                    " EndPosition=" & Chr(34) & giObject.EndPosition & Chr(34) & strTarget & ">" _
                    & giObject.LinkedPath & "</InlineShape>"
            Case 3 ' wdInlineShapePicture
                strResult = "<InlineShape Type=" & Chr(34) & "UnknownPictureFormat" & Chr(34) & _
                    " StartPosition=" & Chr(34) & giObject.StartPosition & Chr(34) & _
                    " EndPosition=" & Chr(34) & giObject.EndPosition & Chr(34) & strTarget & "/>"
        End Select
        
    '
    ElseIf giObject.WordShapeType = giShape Then
    
        ' Handle all shapes but AutoShapes.
        If giObject.GraphicType <> msoAutoShape Then
            strResult = "<Shape Type=" & Chr(34) & giObject.GraphicType & Chr(34) & _
                " AnchorPosition=" & Chr(34) & giObject.StartPosition & Chr(34) & strTarget & "/>"
        
        ' Handle AutoShapes.
        ElseIf giObject.GraphicType = msoAutoShape Then
            strResult = "<Shape Type=" & Chr(34) & giObject.GraphicType & Chr(34) & _
            " AnchorPosition=" & Chr(34) & giObject.StartPosition & Chr(34) & _
            " AutoShapeType=" & Chr(34) & giObject.GraphicSubType & Chr(34) & strTarget & "/>"
        End If
    End If
    
    WriteGraphicInfo = strResult

End Function
Function ParseForGraphics(WordDocument As Word.Document, dgiCurrent As DocumentGraphicInformation, curRange As Word.Range, Optional blnStyledText As Boolean = False) As String
' Parse a range for graphics.
    
    Dim giCurrent As GraphicInstance
    Dim giNext As GraphicInstance
    Dim strTemp As String 'This stores the string containing the XML representation of a graphic.
    
    With dgiCurrent
        ' The current range is a graphic only.
        If .GraphicInstances(1).StartPosition = curRange.Start And _
        .GraphicInstances(1).EndPosition = curRange.End Then
            ParseForGraphics = WriteGraphicInfo(giObject:=.GraphicInstances(1))
        ' The current range doesn't contain any graphics.
        ElseIf .GraphicInstances(1).StartPosition < curRange.Start Or _
            .GraphicInstances(1).StartPosition >= curRange.End Then
            If blnStyledText Then
                ParseForGraphics = CleanString(strDirty:=curRange.Text, TextTags:=False)
            Else
                ParseForGraphics = CleanString(strDirty:=curRange.Text, TextTags:=True)
            End If
        Else
            ' The current range contains text and graphic.
            ' This writes out range text if it appears before the graphic.
            If .GraphicInstances(1).StartPosition > curRange.Start Then
                If blnStyledText Then
                    strTemp = CleanString(strDirty:=WordDocument.Range(Start:=curRange.Start, End:=dgiCurrent.GraphicInstances(1).StartPosition), TextTags:=False)
                Else
                    strTemp = CleanString(strDirty:=WordDocument.Range(Start:=curRange.Start, End:=dgiCurrent.GraphicInstances(1).StartPosition), TextTags:=True)
                End If
            End If
            
            ' Handle graphics and text groups in this range.
            For Each giCurrent In dgiCurrent.GraphicInstances
                If giCurrent.Index <> dgiCurrent.GraphicInstances.Count Then
                    Set giNext = dgiCurrent.GraphicInstances(giCurrent.Index + 1)
                    
                    If giCurrent.EndPosition = giNext.StartPosition Then
                        strTemp = strTemp & WriteGraphicInfo(giObject:=giCurrent)
                    Else
                        If blnStyledText Then
                            strTemp = strTemp & WriteGraphicInfo(giObject:=giCurrent) & _
                                CleanString(strDirty:=WordDocument.Range(Start:=giCurrent.EndPosition, End:=giNext.StartPosition), TextTags:=False)
                        Else
                            strTemp = strTemp & WriteGraphicInfo(giObject:=giCurrent) & _
                                CleanString(strDirty:=WordDocument.Range(Start:=giCurrent.EndPosition, End:=giNext.StartPosition), TextTags:=True)
                        End If
                    End If
                Else
                    If giCurrent.EndPosition > curRange.End Then
                        strTemp = strTemp & WriteGraphicInfo(giObject:=giCurrent)
                    Else
                        If blnStyledText Then
                            strTemp = strTemp & WriteGraphicInfo(giObject:=giCurrent) & _
                                CleanString(strDirty:=WordDocument.Range(Start:=giCurrent.EndPosition, End:=curRange.End), TextTags:=False)
                        Else
                            strTemp = strTemp & WriteGraphicInfo(giObject:=giCurrent) & _
                                CleanString(strDirty:=WordDocument.Range(Start:=giCurrent.EndPosition, End:=curRange.End), TextTags:=True)
                        End If
                    End If
                End If
            Next
            ParseForGraphics = strTemp
        End If
    End With

End Function

Function ParseTable(tblCurrent As Word.Table, Optional DocStyles As DocumentStyleInformation, Optional Graphics As DocumentGraphicInformation) As String
' Write table and cells XML.
    Dim strTemp As String 'This is the current XML string being built.
    Dim intRowCount As Integer 'This is the current row in the table.
    Dim rowCurrent As Word.Row
    Dim celCurrent As Word.Cell
    Dim parCurrent As Word.Paragraph
    Dim blnNewTable As Boolean 'This is a flag used to determine if a new table has been encountered (a nested table).
        
On Error GoTo ErrHandler
    strTemp = "<Paragraph><Table Rows=" & Chr(34) & tblCurrent.Rows.Count & Chr(34) & " Columns=" & Chr(34) & tblCurrent.Columns.Count & Chr(34) & ">"
    
    ' Loop through each row.
    For intRowCount = 1 To tblCurrent.Rows.Count
        Set rowCurrent = tblCurrent.Rows(intRowCount)
        
        strTemp = strTemp & "<Row>"
        
        ' Loopsthrough each cell in current row.
        For Each celCurrent In rowCurrent.Cells
            
            ' Call cell parser.
            strTemp = strTemp & "<Cell>" & ParseCell(celActive:=celCurrent, DocStyles:=DocStyles, Graphics:=Graphics) & "</Cell>"
        Next
        strTemp = strTemp & "</Row>"
    Next
    strTemp = strTemp & "</Table></Paragraph>"
ParseTable = strTemp

Exit Function

ErrHandler:
    Select Case Err.Number
        Case Else
            MsgBox prompt:="An error occurred", _
                Buttons:=vbOKOnly, _
                Title:="Error!"
            Exit Function
    End Select

End Function

Function ParseCell(celActive As Word.Cell, Optional DocStyles As DocumentStyleInformation, Optional Graphics As DocumentGraphicInformation) As String
' Parses contents of individual cells.
    Dim blnNotInTable As Boolean 'This is a flag to determine if the table is done.
    Dim strTemp As String 'This is the current XML string being built.
    Dim intCellTableParagraphsCount As Integer 'This is the count of paragraphs in the cell.
    Dim intTableCount As Integer 'This is used to keep track of what table is being processed.
    Dim blnProcessTable As Boolean 'This is a flag to determine whether the routine is currently processing a table.
    Dim parCurrent As Word.Paragraph
    Dim tblCurrent As Word.Table
    Dim celCurrent As Word.Cell
    Dim intCounter As Integer 'This is used a counter in a For Next loop.
    
    blnProcessTable = True
    blnNotInTable = True
    intTableCount = 0
            
        ' Checks to see if cell contains a child table.
        For Each parCurrent In celActive.Range.Paragraphs
            If blnNotInTable = False Then
                If intCounter = intCellTableParagraphsCount Then
                    blnProcessTable = True
                    blnNotInTable = True
                Else
                    intCounter = intCounter + 1
                    blnProcessTable = False
                End If
            End If
            
            
            If blnProcessTable Then
                For Each tblCurrent In celActive.Tables
                    For Each celCurrent In tblCurrent.Range.Cells
                        If parCurrent.Range = celCurrent.Range Then
                            intCounter = 1
                            intTableCount = intTableCount + 1
                            intCellTableParagraphsCount = tblCurrent.Range.Paragraphs.Count
                            blnNotInTable = False
                            If parCurrent.Range.Cells(1).NestingLevel > celActive.NestingLevel And blnProcessTable Then
                                strTemp = strTemp & ParseTable(tblCurrent:=celActive.Tables(intTableCount), DocStyles:=DocStyles, Graphics:=Graphics)
                                blnProcessTable = False
                            End If
                            Exit For
                        End If
                    Next
                    If blnProcessTable = False Then
                        blnNotInTable = False
                        Exit For
                    End If
                Next
            End If
            
            ' If cell is not in table, parses range as a paragraph.
            If blnNotInTable Then
                strTemp = strTemp & ParseParagraph(parCounter:=parCurrent, DocStyles:=DocStyles, Graphics:=Graphics)
            End If
        Next
ParseCell = strTemp
End Function

Function ParseParagraph(parCounter As Word.Paragraph, Optional DocStyles As DocumentStyleInformation, Optional Graphics As DocumentGraphicInformation) As String
' Parses individual paragraphs for style and graphic information.

Dim dsiCurrent As DocumentStyleInformation
Dim dgiCurrent As DocumentGraphicInformation
Dim siCurrent As StyleInstance
Dim siNext As StyleInstance
Dim giCurrent As GraphicInstance
Dim giNext As GraphicInstance
Dim strTemp As String 'This is the current XML string being built.

    Set dsiCurrent = New DocumentStyleInformation
    Set dgiCurrent = New DocumentGraphicInformation
    ' Get a DocumentStyleInformation object for all the character styles
    ' that apply to this paragraph.
    For Each siCurrent In DocStyles.StyleInstances
        With siCurrent
            If .StartPosition >= parCounter.Range.Start And _
            .StartPosition < parCounter.Range.End Then
                dsiCurrent.StyleInstances.Add StyleName:=.StyleName, _
                    StartPosition:=.StartPosition, _
                    EndPosition:=.EndPosition, _
                    StyleType:=.StyleType, _
                    Target:=.Target
            End If
        End With
    Next
    
    ' Get a DocumentGraphicInformation object for all the graphics
    ' contained in this paragraph.
    If objConverter.GraphicsInfo = xmlWriteAllGraphics Or objConverter.GraphicsInfo = xmlWriteLinkedGraphics Then
        For Each giCurrent In Graphics.GraphicInstances
            With giCurrent
                If .StartPosition >= parCounter.Range.Start And _
                .StartPosition < parCounter.Range.End Then
                    dgiCurrent.GraphicInstances.Add WordShapeType:=.WordShapeType, _
                        StartPosition:=.StartPosition, _
                        EndPosition:=.EndPosition, _
                        GraphicType:=.GraphicType, _
                        LinkedPath:=.LinkedPath, _
                        GraphicSubType:=.GraphicSubType, _
                        OLEClass:=.OLEClass, _
                        Target:=.Target
                End If
            End With
        Next
    End If

    If dsiCurrent.StyleInstances.Count > 0 Then
        Set siCurrent = dsiCurrent.StyleInstances(1)

        If parCounter.Range.Start <> siCurrent.StartPosition Then
            strTemp = "<Paragraph StyleName=" & Chr(34) & _
                Replace(parCounter.Style, Chr(32), "") & Chr(34) & ">"

            If dgiCurrent.GraphicInstances.Count > 0 Then
                strTemp = strTemp & ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(Start:=parCounter.Range.Start, End:=siCurrent.StartPosition))
            Else
                strTemp = strTemp & CleanString(strDirty:=WordDocument.Range(Start:=parCounter.Range.Start, End:=siCurrent.StartPosition), TextTags:=True)
            End If
        Else
            strTemp = "<Paragraph StyleName=" & Chr(34) & Replace(Expression:=parCounter.Style, Find:=Chr(32), Replace:="") & Chr(34) & ">"
        End If

        For Each siCurrent In dsiCurrent.StyleInstances
            ' Print styled text and text until it gets to the last style.
            If siCurrent.Index <> dsiCurrent.StyleInstances.Count Then
                Set siNext = dsiCurrent.StyleInstances(siCurrent.Index + 1)

                ' Print styled text and paragraph text up to next style start.
                If siCurrent.EndPosition <> siNext.StartPosition Then
                    If siCurrent.Target <> Empty Then
                        If dgiCurrent.GraphicInstances.Count > 0 Then
                            strTemp = strTemp & _
                                "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & _
                                " target=" & Chr(34) & siCurrent.Target & Chr(34) & ">" & _
                                 ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(Start:=siCurrent.StartPosition, End:=siCurrent.EndPosition), blnStyledText:=True) & _
                                "</StyledText>" & ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(Start:=siCurrent.EndPosition, End:=siNext.StartPosition), blnStyledText:=False)
                        Else
                            strTemp = strTemp & _
                                "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & _
                                " target=" & Chr(34) & siCurrent.Target & Chr(34) & ">" & _
                                 CleanString(strDirty:=WordDocument.Range(Start:=siCurrent.StartPosition, End:=siCurrent.EndPosition), TextTags:=False) & _
                                "</StyledText>" & CleanString(strDirty:=WordDocument.Range(Start:=siCurrent.EndPosition, End:=siNext.StartPosition), TextTags:=True)
                        End If
                    Else
                        If dgiCurrent.GraphicInstances.Count > 0 Then
                            strTemp = strTemp & _
                                "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & ">" & _
                                 ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(Start:=siCurrent.StartPosition, End:=siCurrent.EndPosition), blnStyledText:=True) & _
                                "</StyledText>" & ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(Start:=siCurrent.EndPosition, End:=siNext.StartPosition))
                        Else
                            strTemp = strTemp & _
                                "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & ">" & _
                                 CleanString(WordDocument.Range(siCurrent.StartPosition, siCurrent.EndPosition), False) & _
                                "</StyledText>" & CleanString(strDirty:=WordDocument.Range(siCurrent.EndPosition, siNext.StartPosition), TextTags:=True)
                        End If
                    End If
                ' Print styled text only.
                Else
                    If siCurrent.Target <> Empty Then
                        If dgiCurrent.GraphicInstances.Count > 0 Then

                            strTemp = strTemp & _
                                "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & _
                                " target=" & Chr(34) & siCurrent.Target & Chr(34) & ">" & _
                                ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(Start:=siCurrent.StartPosition, End:=siCurrent.EndPosition), blnStyledText:=True) & _
                                "</StyledText>"
                        Else
                            strTemp = strTemp & _
                                "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & _
                                " target=" & Chr(34) & siCurrent.Target & Chr(34) & ">" & _
                                CleanString(strDirty:=WordDocument.Range(siCurrent.StartPosition, siCurrent.EndPosition), TextTags:=True) & _
                                "</StyledText>"
                        End If
                    Else
                        If dgiCurrent.GraphicInstances.Count > 0 Then

                            strTemp = strTemp & "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & ">" & _
                                ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(Start:=siCurrent.StartPosition, End:=siCurrent.EndPosition), blnStyledText:=True) & _
                                "</StyledText>"
                        Else
                            strTemp = strTemp & "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & ">" & _
                                CleanString(strDirty:=WordDocument.Range(Start:=siCurrent.StartPosition, End:=siCurrent.EndPosition), TextTags:=False) & _
                                "</StyledText>"
                        End If
                    End If
                End If
            Else
                If siCurrent.Target <> Empty Then
                    If dgiCurrent.GraphicInstances.Count > 0 Then
                        strTemp = strTemp & _
                            "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & _
                            " target=" & Chr(34) & siCurrent.Target & Chr(34) & ">" & _
                            ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(siCurrent.StartPosition, siCurrent.EndPosition), blnStyledText:=True) & _
                            "</StyledText>" & CleanString(strDirty:=WordDocument.Range(siCurrent.EndPosition, parCounter.Range.End), TextTags:=True)
                    Else
                        strTemp = strTemp & _
                            "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & _
                            " target=" & Chr(34) & siCurrent.Target & Chr(34) & ">" & _
                            CleanString(WordDocument.Range(Start:=siCurrent.StartPosition, End:=siCurrent.EndPosition), False) & _
                            "</StyledText>" & CleanString(strDirty:=WordDocument.Range(Start:=siCurrent.EndPosition, End:=parCounter.Range.End), TextTags:=True)
                    End If
                Else
                    If dgiCurrent.GraphicInstances.Count > 0 Then
                        strTemp = strTemp & _
                            "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & ">" & _
                            ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=WordDocument.Range(siCurrent.StartPosition, siCurrent.EndPosition), blnStyledText:=True) & _
                            "</StyledText>" & CleanString(strDirty:=WordDocument.Range(Start:=siCurrent.EndPosition, End:=parCounter.Range.End), TextTags:=True)
                    Else
                        strTemp = strTemp & "<StyledText StyleName=" & Chr(34) & siCurrent.StyleName & Chr(34) & ">" & _
                            CleanString(strDirty:=WordDocument.Range(Start:=siCurrent.StartPosition, End:=siCurrent.EndPosition), TextTags:=False) & _
                            "</StyledText>" & CleanString(strDirty:=WordDocument.Range(Start:=siCurrent.EndPosition, End:=parCounter.Range.End), TextTags:=True)
                    End If
                End If
            End If
        Next

        Set siCurrent = dsiCurrent.StyleInstances(dsiCurrent.StyleInstances.Count)
    Else
        If dgiCurrent.GraphicInstances.Count > 0 Then
                strTemp = _
                    "<Paragraph StyleName=" & Chr(34) & Replace(parCounter.Style, Chr(32), "") & Chr(34) & ">" & _
                    ParseForGraphics(WordDocument:=WordDocument, dgiCurrent:=dgiCurrent, curRange:=parCounter.Range)
        Else
                strTemp = _
                    "<Paragraph StyleName=" & Chr(34) & Replace(parCounter.Style, Chr(32), "") & Chr(34) & ">" & _
                    CleanString(strDirty:=parCounter.Range, TextTags:=True)
        End If
    End If
    strTemp = strTemp & "</Paragraph>"
    Set dsiCurrent = Nothing
    Set dgiCurrent = Nothing
    ParseParagraph = strTemp
End Function

Function GetCSSArray(docTemp As Word.Document) As Variant
' Get an array of the CSS descriptions created when a Word document
' is output to HTML.  Use the HTML DOM to access the style descriptions.
    
    Dim colStylesheets As HTMLStyleSheetsCollection
    Dim htmStyleSheet As HTMLStyleSheet
    Dim colStyleRules As HTMLStyleSheetRulesCollection
    Dim HTMLDOM As New HTMLDocument
    Dim ieTemp As InternetExplorer
    Dim intCounter As Integer 'This is a counter used to loop through a Rules collection.
    Dim intCounter2 As Integer 'This is used as a counter in a For Next loop.
    Dim strName As String 'This stores the name of the style.
    Dim strPath As String 'This stores the path of the HTML document.
    Dim varTempArray As Variant 'This stores an array of CSS style information.
    Dim blnStyleAlreadyAdded As Boolean 'This represents whether the style has been added to the array.
    Dim intStyleCounter As Integer 'This is used to loop through the array.
    Dim intCollectionCounter As Integer 'This is a counter used to loop through a StyleSheets collection.
    
    ' Save temporary document as HTML.
    docTemp.SaveAs "TempDocForCSS", wdFormatHTML
    strPath = docTemp.FullName
    docTemp.Close False
    Set ieTemp = New InternetExplorer
    ieTemp.Visible = False
    
    ' Open temporary HTML document in IE.
    ieTemp.navigate strPath
    
    ' Keep code from continuing until the document is
    ' completely loaded in IE.
    Do While ieTemp.readyState <> READYSTATE_COMPLETE
        DoEvents
    Loop
        
    Set HTMLDOM = ieTemp.Document
    Set colStylesheets = HTMLDOM.styleSheets
    
    ' Loop through all the CSS styles.
    For intCollectionCounter = 0 To colStylesheets.length - 1
    
        Set colStyleRules = colStylesheets.Item(intCollectionCounter).rules
        For intCounter = 0 To colStyleRules.length - 1
            With colStyleRules.Item(intCounter)
                strName = Right(String:=.selectorText, length:=Len(.selectorText) - InStr(1, .selectorText, "."))
                
                ' Remove Office specific style names.
                If Left(strName, 3) = "Mso" Then
                    strName = Right(strName, Len(strName) - 3)
                End If
                ' Default heading styles are named H#, change their names back to Heading#.
                If Len(strName) = 2 And _
                    Left(strName, 1) = "H" Then
                    strName = "Heading " & Right(strName, 1)
                End If
                
                If intCounter > 0 Then
                    For intStyleCounter = 1 To UBound(varTempArray, 2)
                        If varTempArray(1, intStyleCounter) = strName Then
                            blnStyleAlreadyAdded = True
                            Exit For
                        Else
                            blnStyleAlreadyAdded = False
                        End If
                    Next
                End If
                If blnStyleAlreadyAdded = False Then
                    If intCounter = 0 Then
                        ReDim varTempArray(1 To 2, 1 To 1)
                    Else
                        ReDim Preserve varTempArray(1 To 2, 1 To UBound(varTempArray, 2) + 1)
                    End If
                    varTempArray(1, UBound(varTempArray, 2)) = strName
                    varTempArray(2, UBound(varTempArray, 2)) = .Style.cssText
                End If
            End With
            
        
        Next
        Set colStyleRules = Nothing
    Next
    ' Return array containing CSS styles.
    GetCSSArray = varTempArray
    
    ' Delete the temporary HTML document.
    Kill strPath
    
End Function
