Attribute VB_Name = "XMLUtils"
Option Explicit

Private Const scCLASS_NAME = "GlobalWatches.HTTPUtils."
Private Const scComponentName = "GlobalWatches"
    
'********************************************************************************
' METHOD NAME:  FormatXMLDoc
'
' INPUTS:       poXMLDoc    an XML document object
'               ptXSLFile   full filename of an XSLT file to use to tranform the
'                           XML document
'
' RETURNS:      string containing the result of the transformation
'
' PURPOSE:      Transform an XML document using an XSLT stylesheet
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Public Function FormatXMLDoc(poXMLDoc As MSXML2.DOMDocument, ptXSLFile As String, Optional parrParams As Variant) As String
    On Error GoTo ErrHandler
    
    Dim oResultXMLDoc As MSXML2.DOMDocument
    Set oResultXMLDoc = TransformXMLDoc(poXMLDoc, ptXSLFile, parrParams)
    FormatXMLDoc = oResultXMLDoc.xml
    Set oResultXMLDoc = Nothing

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "FormatXMLDoc"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "FormatXMLDoc : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function
'********************************************************************************
' METHOD NAME:  TransformXMLDoc
'
' INPUTS:       poXMLDoc    an XML document object
'               ptXSLFile   full filename of an XSLT file to use to tranform the
'                           XML document
'
' RETURNS:      a DON document that represents the transformation
'
' PURPOSE:      Transform an XML document using an XSLT stylesheet
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       10/Dec/2000 DW          Original
'
'********************************************************************************

Public Function TransformXMLDoc(poXMLDoc As MSXML2.DOMDocument, ptXSLFile As String, Optional parrParams As Variant) As MSXML2.DOMDocument
    On Error GoTo ErrHandler
    
    Dim oXSLDoc As MSXML2.FreeThreadedDOMDocument
    Dim oResultXMLDoc As New MSXML2.DOMDocument
    
    Set oXSLDoc = LoadXMLFromFileFree(ptXSLFile)
    oResultXMLDoc.async = False
    ' we do not need to validate output docs
    oResultXMLDoc.validateOnParse = False
    oResultXMLDoc.resolveExternals = False
    
    If IsMissing(parrParams) Then
        poXMLDoc.transformNodeToObject oXSLDoc, oResultXMLDoc
    Else
        TransformXMLWithParams poXMLDoc, oXSLDoc, oResultXMLDoc, parrParams
    End If
    
    ValidateXML oResultXMLDoc, "TransformXMLDoc"
    
    Set TransformXMLDoc = oResultXMLDoc
    Set oXSLDoc = Nothing

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "TransformXMLDoc"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "TransformXMLDoc : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function


'********************************************************************************
' METHOD NAME:  FormatXMLFromFile
'
' INPUTS:       ptXMLFile   full filename of an XML file
'               ptXSLFile   full filename of an XSLT file to use to tranform the
'                           XML document
'
' RETURNS:      string containing the result of the transformation
'
' PURPOSE:      Transform an XML document using an XSLT stylesheet
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Public Function FormatXMLFromFile(ptXMLFile As String, ptXSLFile As String, Optional parrParams As Variant) As String
    Dim oResultXMLDoc As MSXML2.DOMDocument

    On Error GoTo ErrHandler
    
    Set oResultXMLDoc = TransformXMLFromFile(ptXMLFile, ptXSLFile, parrParams)
    FormatXMLFromFile = oResultXMLDoc.xml
    Set oResultXMLDoc = Nothing

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "FormatXMLFromFile"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "FormatXMLFromFile : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function

'********************************************************************************
' METHOD NAME:  TransformXMLFromFile
'
' INPUTS:       ptXMLFile   full filename of an XML file
'               ptXSLFile   full filename of an XSLT file to use to tranform the
'                           XML document
'
' RETURNS:      a DON document that represents the transformation
'
' PURPOSE:      Transform an XML document using an XSLT stylesheet
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       10/Dec/2000 DW          Original
'
'********************************************************************************

Public Function TransformXMLFromFile(ptXMLFile As String, ptXSLFile As String, Optional parrParams As Variant) As MSXML2.DOMDocument
    On Error GoTo ErrHandler
    
    Dim oXMLDoc As MSXML2.DOMDocument
    Dim oXSLDoc As MSXML2.FreeThreadedDOMDocument
    Dim oResultXMLDoc As New MSXML2.DOMDocument
    
    Set oXMLDoc = LoadXMLFromFile(ptXMLFile)
    Set oXSLDoc = LoadXMLFromFileFree(ptXSLFile)
    oResultXMLDoc.async = False
    ' we do not need to validate output docs
    oResultXMLDoc.validateOnParse = False
    oResultXMLDoc.resolveExternals = False
    
    If IsMissing(parrParams) Then
        oXMLDoc.transformNodeToObject oXSLDoc, oResultXMLDoc
    Else
        TransformXMLWithParams oXMLDoc, oXSLDoc, oResultXMLDoc, parrParams
    End If
    
    ValidateXML oResultXMLDoc, "TransformXMLFromFile"
    
    Set TransformXMLFromFile = oResultXMLDoc
    Set oXMLDoc = Nothing
    Set oXSLDoc = Nothing

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "TransformXMLFromFile"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "TransformXMLFromFile : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function


'********************************************************************************
' METHOD NAME:  FormatXMLFromURL
'
' INPUTS:       ptXMLURL    URL which will return XML data
'               ptXSLFile   full filename of an XSLT file to use to tranform the
'                           XML document
'               bDirect     true if the URL sets the content type of the returned
'                           data to text/xml, false otherwise
'
' RETURNS:      string containing the result of the transformation
'
' PURPOSE:      Transform an XML document using an XSLT stylesheet.  The routine
'               is more efficient is the bDirect parameter can be set to true.
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Public Function FormatXMLFromURL(ptXMLURL As String, ptXSLFile As String, bDirect As Boolean, Optional parrParams As Variant) As String
    On Error GoTo ErrHandler
    
    Dim oResultXMLDoc As MSXML2.DOMDocument
    Set oResultXMLDoc = TransformXMLFromURL(ptXMLURL, ptXSLFile, bDirect, parrParams)
    FormatXMLFromURL = oResultXMLDoc.xml
    Set oResultXMLDoc = Nothing

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "FormatXMLFromURL"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "FormatXMLFromURL : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function

'********************************************************************************
' METHOD NAME:  TransformXMLFromURL
'
' INPUTS:       ptXMLURL    URL which will return XML data
'               ptXSLFile   full filename of an XSLT file to use to tranform the
'                           XML document
'               bDirect     true if the URL sets the content type of the returned
'                           data to text/xml, false otherwise
'
' RETURNS:      a DON document that represents the transformation
'
' PURPOSE:      Transform an XML document using an XSLT stylesheet
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       10/Dec/2000 DW          Original
'
'********************************************************************************

Public Function TransformXMLFromURL(ptXMLURL As String, ptXSLFile As String, bDirect As Boolean, Optional parrParams As Variant) As MSXML2.DOMDocument
    On Error GoTo ErrHandler
    
    Dim oXMLDoc As MSXML2.DOMDocument
    Dim oXSLDoc As MSXML2.FreeThreadedDOMDocument
    Dim oResultXMLDoc As New MSXML2.DOMDocument
    
    Set oXMLDoc = LoadXMLFromURL(ptXMLURL, bDirect)
    Set oXSLDoc = LoadXMLFromFileFree(ptXSLFile)
    oResultXMLDoc.async = False
    ' we do not need to validate output docs
    oResultXMLDoc.validateOnParse = False
    oResultXMLDoc.resolveExternals = False
    
    If IsMissing(parrParams) Then
        oXMLDoc.transformNodeToObject oXSLDoc, oResultXMLDoc
    Else
        TransformXMLWithParams oXMLDoc, oXSLDoc, oResultXMLDoc, parrParams
    End If
    
    ValidateXML oResultXMLDoc, "TransformXMLFromURL"
    
    Set TransformXMLFromURL = oResultXMLDoc
    Set oXMLDoc = Nothing
    Set oXSLDoc = Nothing

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "TransformXMLFromURL"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "TransformXMLFromURL : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function

'********************************************************************************
' METHOD NAME:  LoadXMLFromURL
'
' INPUTS:       ptXMLURL    URL which will return XML data
'               bDirect     true if the URL sets the content type of the returned
'                           data to text/xml, false otherwise
'
' RETURNS:      XML document object
'
' PURPOSE:      Load and parse an XML document from an HTTP server.  The routine
'               is more efficient is the bDirect parameter can be set to true.
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Public Function LoadXMLFromURL(ptXMLURL As String, bDirect As Boolean) As MSXML2.DOMDocument
    Dim oXMLDoc As New MSXML2.DOMDocument
    
    On Error GoTo ErrHandler
    
    If bDirect Then
        Set oXMLDoc = GetXMLURL(ptXMLURL)
    Else
        Dim oXMLStream As Variant

        oXMLDoc.async = False
        Set oXMLStream = GetStreamURL(ptXMLURL)
        oXMLDoc.Load (oXMLStream)
        
        Set oXMLStream = Nothing
    End If
    
    ValidateXML oXMLDoc, ptXMLURL
    
    Set LoadXMLFromURL = oXMLDoc

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "LoadXMLFromURL"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "LoadXMLFromURL : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function


'********************************************************************************
' METHOD NAME:  LoadXMLFromFile
'
' INPUTS:       ptXMLFile   full filename of an XML file
'
' RETURNS:      XML document object
'
' PURPOSE:      Load and parse an XML document from a file.
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Public Function LoadXMLFromFile(ptXMLFile As String) As MSXML2.DOMDocument
    On Error GoTo ErrHandler
    
    Dim oXMLDoc As New MSXML2.DOMDocument

    oXMLDoc.async = False
    oXMLDoc.Load (ptXMLFile)
    
    ValidateXML oXMLDoc, ptXMLFile
    
    Set LoadXMLFromFile = oXMLDoc

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "LoadXMLFromFile"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "LoadXMLFromFile : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function

Public Function LoadXMLFromFileFree(ptXMLFile As String) As MSXML2.FreeThreadedDOMDocument
    On Error GoTo ErrHandler
    
    Dim oXMLDoc As New MSXML2.FreeThreadedDOMDocument

    oXMLDoc.async = False
    oXMLDoc.Load (ptXMLFile)
    
    ValidateXML oXMLDoc, ptXMLFile
    
    Set LoadXMLFromFileFree = oXMLDoc

ExitMethod:
    Exit Function
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "LoadXMLFromFileFree"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "LoadXMLFromFileFree : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Function


'********************************************************************************
' METHOD NAME:  ValidateXML
'
' INPUTS:       oXMLDoc     an XML document object
'               strXMLsrc   source of the XML data
'
' RETURNS:
'
' PURPOSE:      Checks that the document contains valid data.  If the data could
'               not be parsed then throws an error.
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       21/06/2000  LTC         Original
'
'********************************************************************************

Private Sub ValidateXML(oXMLDoc As MSXML2.DOMDocument, strXMLsrc As String)
    If oXMLDoc.parseError.errorCode <> 0 Then
        Err.Raise oXMLDoc.parseError.errorCode, strXMLsrc, "XMLError: " & oXMLDoc.parseError.reason & " : Line:" & CStr(oXMLDoc.parseError.Line) & ", Pos:" & CStr(oXMLDoc.parseError.linepos)
    End If
End Sub

Public Sub TransformXMLWithParams(oXMLDoc As MSXML2.DOMDocument, oXSLDoc As MSXML2.FreeThreadedDOMDocument, oResultXMLDoc As MSXML2.DOMDocument, parrParams As Variant)
    On Error GoTo ErrHandler
    
    ' the object that will do the transform
    Dim oXSLTrans As New MSXML2.XSLTemplate
    Set oXSLTrans.stylesheet = oXSLDoc
    
    ' we need to access the processor to set the input,output and params
    Dim oXSLProcessor As MSXML2.IXSLProcessor
    Set oXSLProcessor = oXSLTrans.createProcessor
    
    ' setup the input (populated) and output (empty) XML DOM objects
    oXSLProcessor.input = oXMLDoc
    oXSLProcessor.output = oResultXMLDoc
    
    ' param data is an array where
    '       elem 0 = name of param
    '       elem 1 = value of param
    '       elem 2 = name of param
    '       elem 3 = value of param etc...
    Dim count As Integer
    For count = 0 To UBound(parrParams) - 1 Step 2
        If count + 1 <= UBound(parrParams) Then
            oXSLProcessor.addParameter parrParams(count), CStr(parrParams(count + 1))
        End If
    Next
    
    ' perform the transformation
    oXSLProcessor.Transform
    
ExitMethod:
    Exit Sub
    
ErrHandler:
    Dim strErrSource, strErrDesc As String
    Dim lngErrNum As Long

    strErrDesc = Err.Description
    lngErrNum = Err.Number
    strErrSource = Err.Source
    
    On Error Resume Next
    
    If strErrSource = scComponentName Then
        strErrSource = scCLASS_NAME & "TransformXMLWithParams"
    ElseIf InStr(strErrSource, scCLASS_NAME) = 0 Then
        strErrSource = scCLASS_NAME & "TransformXMLWithParams : " & strErrSource
    End If
    
    Err.Raise lngErrNum, strErrSource, strErrDesc
    Resume ExitMethod
End Sub

'********************************************************************************
' METHOD NAME:  GetNodeText
'
' INPUTS:       oContextNode     an XML DOM Node
'               strXPath         an XPath query relative to oContextNode
'
' RETURNS:      The text value of the node that the strXPath query references
'
' PURPOSE:      This routine should be used rather than writing oNode("@attr").text
'               if the attr (or element) is optional
'               It checks for the existence of the node before getting its text
'               and returns an empty string if the node referenced by the XPath
'               query does not exist
'
' DEVELOPMENT AND MAINTENANCE HISTORY:
'
' Version   Date        Initials    Description
' =======   ==========  ========    ===========
' 1.0       10/Dec/2000 DW          Original
'
'********************************************************************************

Public Function GetNodeText(oContextNode As MSXML2.IXMLDOMNode, strXPath As String) As String
    GetNodeText = ""
    Dim oTextNode As MSXML2.IXMLDOMNode
    Set oTextNode = oContextNode.selectSingleNode(strXPath)
    If Not oTextNode Is Nothing Then
        GetNodeText = oTextNode.Text
    End If
End Function


Public Function MapXSLFileName(ptXSLFile As String) As String
    MapXSLFileName = App.Path & "/XSL/" & ptXSLFile
End Function


Public Function MapXMLFileName(ptXMLFile As String) As String
    MapXMLFileName = App.Path & "/XML/" & ptXMLFile
End Function

